<?php

namespace Drupal\group_custom_form_mode_manager\Routing\EventSubscriber;

use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;
use Drupal\form_mode_manager\Routing\EventSubscriber\FormModesSubscriber as BaseSubscriber;

/**
 * Listens to the dynamic route event and add routes using form modes.
 */
class FormModesSubscriber extends BaseSubscriber {

  /**
   * Namespace of transverse entity controller.
   *
   * @var string
   */
  const FORM_MODE_DEFAULT_CONTROLLER = '\Drupal\group_custom_form_mode_manager\Controller\GroupContentFormModeManagerEntityController';

  /**
   * Add one route collection per entity using form modes.
   *
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {
    $this->routeCollection = $collection;
    $FormModes = $this->formModeManager->getAllFormModesDefinitions();
    $entityTypeId = 'group_content';
    if (!empty($FormModes[$entityTypeId])) {
      $this->entityDefinition = $this->entityTypeManager->getDefinition($entityTypeId);
      $this->entityRoutingDefinition = $this->entityRoutingMap->createInstance($entityTypeId, ['entityTypeId' => $entityTypeId]);
      $this->addFormModesRoutes($FormModes[$entityTypeId]);
    }
  }

  /**
   * Add a collection of route per form mode for current entity.
   *
   * Each entity need to define a collection of routes to be used by Drupal.
   * Each `entity operation` is a form handler present for all,
   * ContentEntityType plugins. Each operation represent a Form to be,
   * displayed by formBuilder service in routes controllers.
   * To respect core logic implemented by Drupal we need to add,
   * a route for each operations (add/edit/add_page/admin_add) dynamically and,
   * a transverse controller compatible to use form modes.
   *
   * Each routes generated by `setFormModeCollection` and,
   * `setAddPageCollection` respect the following naming,
   * `ENTITY_ROUTES_BASIS.FORM_MODE_NAME` eg for Node entity :
   *
   * @code
   *  $routes = [
   *    'add_form' => 'node.add.FORM_MODE_NAME',
   *    'edit_form' => 'entity.node.edit_form.FORM_MODE_NAME',
   *    'add_page' => 'form_mode_manager.node.add_page.FORM_MODE_NAME',
   *    'admin_add' => 'node.add.FORM_MODE_NAME',
   *  ];
   * @endcode
   *
   * @param array $form_modes
   *   All form-modes available for specified entity_type_id.
   */
  protected function addFormModesRoutes(array $form_modes) {
    foreach ($form_modes as $form_mode_infos) {
      $this->setFormModeCollection($form_mode_infos, 'group-join');
    }
  }


  /**
   * Generate a routes based on top of given parent routes.
   *
   * @param \Symfony\Component\Routing\Route $parent_route
   *   The route object of entity.
   * @param array $form_mode_infos
   *   The form mode info.
   *
   * @return \Symfony\Component\Routing\Route
   *   Form Mode Manager route to be added on entity collection.
   */
  protected function setRoutes(Route $parent_route, array $form_mode_infos) {
    $route_path = implode('/', [
      $parent_route->getPath(),
      $this->formModeManager->getFormModeMachineName($form_mode_infos['id']),
    ]);
    $route_defaults = array_merge($parent_route->getDefaults(), $this->getFormModeRouteDefaults($parent_route, $form_mode_infos));
    $route_options = array_merge($parent_route->getOptions(), $this->getFormModeRouteOptions($form_mode_infos));
    $parentRequirements = $parent_route->getOptions();
    // This not works and access denied happen, no time to see why :D.
    unset($parentRequirements['_group_permission']);
    $route_requirements = array_merge($parentRequirements, $this->getFormModeRouteRequirements($form_mode_infos));

    return new Route($route_path, $route_defaults, $route_requirements, $route_options);
  }

}
